!<
!create_nfilter
!
!Usage:
!create_nfilter.x  <in.H >out.H  [pars]
!
!Input Parameters:
!
!stdin  - velocity field
!
! Generic Pars
!
!tau - logical [.false.] Whether we should convert to tau
!d_tau- real             Sampling in tau
!n_tau- integer          Number of samples in tau
!verb- logical [.false.] Whether or not to be verbose
!test_it-logical [.false] Whether or not to run a missing data test
!beg1 - integer  [0]      Amount to pad first axis top
!beg2 - integer  [0]      Amount to pad second axis top
!end1 - integer  [0]      Amount to pad first axis bottom
!end2 - integer  [0]      Amount to pad second axis bottom
!
!Type of nfilter
!
!isotropic - logical [.true.]  Whether or not to construct an isotropic filter
!non_stationary-logical [.false.] Whether or not to construct a non-stat filter
!
!Isotropic parameters (iso-.true.)
!
!scale_lap - real        [1.]  What to scale the laplacian to
!npts      - integer     [6]   Number of pts to use for laplacian filter
!
!Steering filters (iso-.false.,non_stat=.false.)
!
!max_amp   - real        [.99]  Maximum amplitude for filter
!ns        - integer     [3]    Not sure (building slopes)
!nwind     - integer     [2]    Size of median smoother
!reflectors- sepfile            Sepfile containing reflector positions
!use_ref   - integer(:)  [all]  Which reflectors to use when finding dips
!use_faults- integer(:)  [none] Reflectors which represent sharp bounaries
!min_amp   - real        [.1]   Minimum amp when close to fault
!range     - real        [n*d/2.] Maximum range of fault influence
!
!
!Non-stationary filtering
!
!ns_j      - integer(2)  [5,5]  Spacing of micro patches
!ns_na     - integer(2)  [5,3]  Size of non-stationary PEF
!ns_center - integer(2)  [3,1]  Location of fixed 1
!ns_eps    - real        [1.]   Epsilon to use when estimating PEF
!ns_niter  - integer     [30]   Number of itterations to do when estimating PEF
!ref_old   - sepfile            Locations of reflectors in current vel model
!ref_new   - sepfile            Location of same refs after migration
!from_initial-logical   [.false.] Do warping off initial velocity and refs
!ref_init  - sepfile            Locations of reflectors initial model
!vel_init  - sepfile            Initial velocity model
!
!
!
!Output Parameters:
!
!stdout      - sepfile      nonstationary PEF
!dips        - sepfile      File containing dips (only steering filters)
!amp         - sepfile      File containing amps (only steering filters)
!
!Description:
!
! Constructs a non stationary filter
!
!>
!-------------------------------------------------
!
!Author: Robert Clapp, ESMB 463, 7230253
!
!Date Created:Mon Mar 20  9:41:02 PST 2000
!
!Purpose:
!
program create_nfilter 
  use sep
  use npef
  use build_filter_mod
  use build_steering_filters
  use warp_mod
  use createnhelixmod
  use mask1
  use pad_mod
  use io_nfilt
  use lapfac
  implicit none
  integer ::  n(2),j(2),center(2),na(2),ncoef,n123,niter,iloc1,iloc2&
    &,nold(2)
  integer :: nuse(2),n_t(2),n_tau,use_faults(100)
  integer :: beg1,end1,beg2,end2,nbig(2),nc,i1,i2,ntau,npts,nuse_ref
  real    :: obig(2),dbig(2),d_tau
  integer   ::  nbig123
  real ::  o(2),d(2),hwidth,eps,o_t(2),d_t(2),ouse(2),duse(2),min_amp
  real :: max_amp,range,oold(2),dold(2)
  character(len=128) :: label,tag_in,tag_ref
  real, pointer :: data(:),covar(:),temp(:,:)
  real, pointer :: map(:,:),velocity(:,:),tempv(:,:)
  logical, pointer :: mask(:)
  logical ::  test_it
  type(nfilter) :: rough,filt
  real :: scale_lap
  logical ::  tau,isotropic,non_stationary,verb
  integer :: use_refs(100),nref,npts,ns,nwind
  integer,external :: getch,auxin
  complex, allocatable :: refs(:,:),faults(:,:)
  real, allocatable :: ref_old(:,:,:),ref_new(:,:,:),p(:,:),amps(:,:)&
    &,velt(:,:),tt(:,:,:)
  real   :: x,z
  logical :: have_faults,from_initial
  integer :: n1f,n2f,n1r,n2r,n1rn,n2rn,i,smooth_wind
!read in parameters
  integer auxputch
  call initpar()
  call doc('/homes/sep/bob/papers/thesis/Src/main/create_nfilter.rs90'&
    &)
  call sep_get_data_axis_par("in",1,n(1),o(1),d(1),label)
  call sep_get_data_axis_par("in",2,n(2),o(2),d(2),label)
  call from_param("tau",tau,.false.)
  call from_param("isotropic",isotropic,.true.)
  call from_param("non_stationary",non_stationary,.false.)
  call from_param("test_it",test_it,.false.)
  call from_param("beg1",beg1,0)
  call from_param("beg2",beg2,0)
  call from_param("end1",end1,0)
  call from_param("end2",end2,0)
  call from_param("verb",verb,.false.)
  call from_param("scale_lap",scale_lap,1.)
  tag_in="in"
  tag_ref="ref_old"
  if (tau) then
    call from_param("dtau",d_tau,d(1))
    call from_param("ntau",n_tau,n(1))
  end if
  if (.not. non_stationary) then
    if (isotropic) then
      call from_param("npts",npts,6)
    else
      call from_param("max_amp",max_amp,.99)
      call from_param("ns",ns,3)
      call from_param("nwind",nwind,2)
      call from_param("hwidth",hwidth,1.5)
      if (-1.eq.auxin("reflectors")) then
        call seperr("must provide reflectors when doing steering filters"&
          &)
      end if
      call from_aux("reflectors","n1",npts)
      call from_aux("reflectors","n2",nref)
      nuse_ref=getch("use_refs","d",use_refs)
      if (nuse_ref.eq.0) then
        do i=1,nref 
          use_refs(i)=i
        end do 
        nuse_ref=nref
      else
        if (any(use_refs(1:nuse_ref) < 1) .or. any(use_refs(1:nuse_ref&
          &)>nref)) then
          call seperr("invalid use refs")
        end if
      end if 
      n2f=getch("use_faults","d",use_faults)
      if (n2f>0) then
        if (verb) then
          write(0,*) "using reflectors ",use_faults(1:n2f)," as&
            & faults"
        end if
        have_faults=.true.
        if (any(use_faults(1:n2f) < 1) .or. any(use_refs(1:n2f)>nref))&
          & then
          call seperr("invalid use faults")
        end if
        n1f=npts
        call from_param("min_amp",min_amp,.1)
        call from_param("range",range,.5*maxval(n*d))
      else
        have_faults=.false.
      end if
    end if
  else
    call from_param("ns_j",j,(/5,5/))
    call from_param("ns_na",na,(/5,3/))
    call from_param("ns_center",center,(/ceiling(na(1)/2.),1/))
    call from_param("ns_hwidth",hwidth,4.)
    call from_param("from_initial",from_initial,.false.)
    call from_param("iloc1",iloc1)
    call from_param("iloc2",iloc2)
    call from_param("ns_eps",eps,0.)
    call from_param("ns_niter",niter,30)
    if (.not. from_initial) then
      if (-1.eq.auxin("ref_old")  .or. -1.eq.auxin("ref_new")) then
        call erexit("must provide ref_old and ref_new when doing non&
          &-stationary")
      end if
    else
      if (auxin("ref_init").eq.-1 .or. auxin("vel_init").eq.-1) then
        call erexit("must provide ref_init and vel_init when doing non&
          &-stationary using initial model")
      end if
    end if 
    if (from_initial) then
      tag_in="vel_init"
      tag_ref="ref_init"
      call sep_get_data_axis_par("vel_init",1,n(1),o(1),d(1),label)
      call sep_get_data_axis_par("vel_init",2,n(2),o(2),d(2),label)
    end if
    call from_aux("ref_new","n1",n1rn)
    call from_aux(tag_ref,"n1",n1r)
    call from_aux(tag_ref,"n2",n2r)
    call from_aux("ref_new","n2",n2rn)
  end if 
  nbig(1)=beg1+end1+n(1)
  nbig(2)=beg2+end2+n(2)
  obig=o-d*(/beg1,beg2/)
  dbig=d
  if (tau) then
!convert vel to tau
    if (verb) then
      write(0,*) "using tau space"
    end if
    allocate(map(nbig(1),nbig(2)))
    n_t=(/beg1+end1+n_tau,nbig(2)/)
    o_t=(/0.-d_tau*beg1,obig(2)/)
    d_t=(/d_tau,d(2)/)
    nuse=n_t
    ouse=o_t
    duse=d_t
  else
    nuse=nbig
    ouse=obig
    duse=dbig
  end if 
  nbig123=product(nuse)
  n123=product(n)
  if (verb) then
    write(0,*) "padding axis 1",beg1,end1," axis2 ",beg2,end2
  end if
  if (.not. non_stationary .and. isotropic) then
!isotropic regularizer
    if (verb) then
      write(0,*) "constructing isotropic regularizer",npts
    end if
    allocate(filt%hlx(1))
    allocate(filt%pch(nbig123))
    filt%pch=1
    filt%hlx(1)=lapfac2(.000001,nuse(1),npts)
    if (scale_lap.ne.0.) then
      filt%hlx(1)%flt=-scale_lap*filt%hlx(1)%flt/sum(filt%hlx(1)%flt)
    end if
  else
    allocate(velocity(n(1),n(2)))
    call pad_init(beg1,beg2,end1,end2)
    call sreed(tag_in,velocity,n123*4)
    call pad_it_2d(velocity)
    write(0,*) "what is going",o_t
    if (tau) then
      write(0,*) "initializing tau conversion",o_t
      call init_tau_z_conv(nbig,obig,dbig,n_t,o_t,d_t)
      call create_tau_map(velocity,map)
    end if
    if (.not. non_stationary) then
!if we are constructing steering filters
      if (verb) then
        write(0,*) "creating steering filter"
      end if
      allocate(refs(npts,nref))
      allocate(p(nuse(1),nuse(2)))
      call sreed("reflectors",refs,size(refs)*8)
      if (tau) then
        call convert_ref_z_to_tau(refs)
      end if
      call srite("junk.H",refs,size(refs)*8)
      call default_steering_filters()	
      call init_steering_filters(n_in=nuse,o_in=ouse,d_in=duse&
        &,use_refs_in=use_refs(1:nuse_ref),ns_in=ns,nwind_in=nwind)
      call build_2d_steer_p_map(refs,p)
      if (verb) then
        write(0,*) "p range",minval(p),maxval(p)
      end if
      allocate(amps(nuse(1),nuse(2)))
      if (have_faults) then
        allocate(faults(n1f,n2f))
        do i2=1,n2f 
          faults(:,i2)=refs(:,use_faults(i2))
        end do 
!				if(tau) call convert_ref_z_to_tau(faults)
        call init_steering_filters(min_amp_in=min_amp,max_amp_in=max_amp&
          &,range_in=range)
        if (verb) then
          write(0,*) "using",n2f," fault reflectors range=",range
        end if
        call build_2d_steer_amp_map(faults,amps)
      else
        amps=max_amp
      end if 
      if (verb) then
        write(0,*) "amp range",minval(amps),maxval(amps)
      end if
      call from_param("smooth_wind",smooth_wind,0)
      if (smooth_wind>0) then
        allocate(tt(size(p,1),size(p,2),1))
        tt(:,:,1)=p
        call gauss_smooth(tt,smooth_wind,smooth_wind,1,p(1))
        p=tt(:,:,1)
        deallocate(tt)
      end if
      if (1.eq.getch("dips","s",label)) then
          if (0.ne. auxputch('n1','d',nuse(1) ,'dips')) then
            call erexit('Trouble writing  n1 to aux dips ')
          end if
          if (0.ne. auxputch('n2','d',nuse(2) ,'dips')) then
            call erexit('Trouble writing  n2 to aux dips ')
          end if
          if (0.ne. auxputch('o1','f',ouse(1) ,'dips')) then
            call erexit('Trouble writing  o1 to aux dips ')
          end if
          if (0.ne. auxputch('o2','f',ouse(2) ,'dips')) then
            call erexit('Trouble writing  o2 to aux dips ')
          end if
          if (0.ne. auxputch('d1','f',duse(1) ,'dips')) then
            call erexit('Trouble writing  d1 to aux dips ')
          end if
          if (0.ne. auxputch('d2','f',duse(2) ,'dips')) then
            call erexit('Trouble writing  d2 to aux dips ')
          end if
        call srite("dips",p,size(p)*4)
      end if
      if (1.eq.getch("amp","s",label)) then
          if (0.ne. auxputch('n1','d',nuse(1) ,'amp')) then
            call erexit('Trouble writing  n1 to aux amp ')
          end if
          if (0.ne. auxputch('n2','d',nuse(2) ,'amp')) then
            call erexit('Trouble writing  n2 to aux amp ')
          end if
          if (0.ne. auxputch('o1','f',ouse(1) ,'amp')) then
            call erexit('Trouble writing  o1 to aux amp ')
          end if
          if (0.ne. auxputch('o2','f',ouse(2) ,'amp')) then
            call erexit('Trouble writing  o2 to aux amp ')
          end if
          if (0.ne. auxputch('d1','f',duse(1) ,'amp')) then
            call erexit('Trouble writing  d1 to aux amp ')
          end if
          if (0.ne. auxputch('d2','f',duse(2) ,'amp')) then
            call erexit('Trouble writing  d2 to aux amp ')
          end if
        call srite("amp",amps,size(amps)*4)
      end if
      call init_steering_filters(hwidth_in=hwidth)
      call create_2d_steer_op(p,amps,filt)
      if (tau) then
        allocate(temp(nuse(1),nuse(2)))
        call convert_field_z_tau(velocity,temp,map)
        deallocate(velocity)
        allocate(velocity(nuse(1),nuse(2)))
        velocity=temp
      end if
    else
!build filters
      call init_warp_2d(nuse,ouse,duse)
      allocate(ref_old(2,n1r,n2r),ref_new(2,n1rn,n2rn))
      if (tau) then
!convert new reflector into tau
        allocate(refs(n1r,n2r))
        call sreed(tag_ref,refs,size(refs)*8)
        call convert_ref_z_to_tau(refs)
        ref_old(1,:,:)=real(refs)
        ref_old(2,:,:)=aimag(refs)
        deallocate(refs)
!now convert initial field to tau
        allocate(temp(nuse(1),nuse(2)))
        call convert_field_z_tau(velocity,temp,map)
        deallocate(velocity)
        allocate(velocity(nuse(1),nuse(2)))
        velocity=temp
!read in current reflector, convert to tau
        if (from_initial) then
          allocate(tempv(n(1),n(2)))
          call sreed("in",tempv,n123*4)
          call pad_it_2d(tempv)
          call create_tau_map(tempv,map)
        end if
        allocate(refs(n1rn,n2rn))
        call sreed("ref_new",refs,size(refs)*8)
        call convert_ref_z_to_tau(refs)
        ref_new(1,:,:)=real(refs)
        ref_new(2,:,:)=aimag(refs)
      else
        call sreed(tag_ref,ref_old,size(ref_old)*4)
        call sreed("ref_new",ref_new,size(ref_new)*4)
      end if 
      call warp_me_2d(ref_old,ref_new,velocity)
      call init_build_filters(nuse,j,center,na,o,d,hwidth,       max_amp&
        &,.false.,.false.,d(1)*5,3.)
      if (verb) then
        write(0,*) na,"=na center=",center
      end if
      if (verb) then
        write(0,*) j,"=j ncoef=",ncoef
      end if
      call build_2d_filter(filt,rough,ncoef)
      rough%pch=1
      deallocate(rough%hlx(1)%flt,rough%hlx(1)%lag)
      rough%hlx(1)=lapfac2(.0000001,nuse(1)/j(1),2)
      rough%hlx(1)%lag=rough%hlx(1)%lag*ncoef
      rough%hlx(1)%flt=-scale_lap*rough%hlx(1)%flt/sum(rough%hlx(1)%flt&
        &)
!find the npef
      velocity=1./velocity
      allocate(covar(nbig123))
      do i2=1,nbig(2)
        do i1=1,nbig(1)
          covar(i1+nbig(1)*(i2-1))=velocity(i1,i2)
        end do
      end do 
      call find_pef(covar,filt,rough,niter,eps,ncoef,.true.)
    end if
  end if 
  if (test_it) then
    allocate(data(nbig123))
    data=0.
    do i2=1,nuse(2),6 
      do i1=1,nuse(1),6 
        data(i1+nuse(1)*(i2-1))=velocity(i1,i2)
      end do
    end do 
    call npolydiv_init(nbig123,filt)
    allocate(mask(nbig123))
    call srite("bbss.H",velocity,size(velocity)*4)
    call srite("tt.H",data,size(velocity)*4)
    mask =  (data .ne. 0.)
    call mask1_init(mask)
    call portion_it(nuse(1)-beg1-end1,nuse(2)-beg2-end2,beg1,beg2,end1&
      &,end2,velocity)
    call portion_it(nuse(1)-beg1-end1,nuse(2)-beg2-end2,beg1,beg2,end1&
      &,end2,data)
      if (0.ne. auxputch('n1','d',nuse(1)-beg1-end1 ,'test')) then
        call erexit('Trouble writing  n1 to aux test ')
      end if
      if (0.ne. auxputch('n2','d',nuse(2)-beg2-end2 ,'test')) then
        call erexit('Trouble writing  n2 to aux test ')
      end if
      if (0.ne. auxputch('n3','d',3 ,'test')) then
        call erexit('Trouble writing  n3 to aux test ')
      end if
      if (0.ne. auxputch('o1','f',o(1) ,'test')) then
        call erexit('Trouble writing  o1 to aux test ')
      end if
      if (0.ne. auxputch('o2','f',o(2) ,'test')) then
        call erexit('Trouble writing  o2 to aux test ')
      end if
      if (0.ne. auxputch('d1','f',duse(1) ,'test')) then
        call erexit('Trouble writing  d1 to aux test ')
      end if
      if (0.ne. auxputch('d2','f',d(2) ,'test')) then
        call erexit('Trouble writing  d2 to aux test ')
      end if
    call nboundn(nuse,filt)
    call solver_prec(mask1_lop,cgstep,niter=50,x=data, dat=data&
      &,prec=npolydiv_lop,nprec=size(mask),eps=0.,verb=.true.)
    call srite("tt.H",data,size(velocity)*4)
    call portion_it(nuse(1)-beg1-end1,nuse(2)-beg2-end2,beg1,beg2,end1&
      &,end2,data)
!write it out
  end if
  call rite_filt("out",filt,nuse,ouse,duse)
end program 
subroutine portion_it(n1,n2,beg1,beg2,end1,end2,array)
  implicit none
  integer :: n1,n2,beg1,beg2,end1,end2
  real :: array((n1+beg1+end1),(n2+beg2+end2))
  call srite("test",array(beg1+1:beg1+n1,beg2+1:beg2+n2),n1*n2*4)
end subroutine  
